/**
 * Called when the "post link" dialog box is created. This retrieves the
 * default values for the form parameters from the URL that specifies the
 * box itself. See for example postCurrentPage() in deliciousOverlay.js.
 */
function loadArgs()
{
    var args=window.arguments[0];

    var url = "";
    var description = "";
    var extended = "";
    var tags = "";
    var pair;

    if (args['url'] != null) url=decodeURIComponent(args['url']);
    if (args['description'] != null) description=decodeURIComponent(args['description']);
    if (args['extended'] != null) extended=decodeURIComponent(args['extended']);
    if (args['tags'] != null) tags=decodeURIComponent(args['tags']);
    if (args['globals'] == null) {
        alert("Del.icio.us extension internal error: globals missing");
        return;
    }
    globals = args['globals'];
    
    if (globals.deliciousService.getUsername().length == 0 || globals.deliciousService.getPassword().length == 0) {
        alert("Please set up an account using the Del.icio.us extension's options first.");
        window.close();
        return;
    }
   
    document.getElementById("url").value = url;
    document.getElementById("description").value = description;
    document.getElementById("extended").value = extended;
    document.getElementById("tags").value = tags;

    var wordsInExtended = extended.replace(/\s+/g, ' ').replace(/^\W+|\W+$/g, '').split(' ').length;
    document.getElementById("extended-row").setAttribute("flex", wordsInExtended/2);

    document.getElementById('extended').focus();
    
    var users = globals.userManager.getUsers();
    var defaultUser = globals.userManager.getDefaultUser().getUsername();
    var postAsPopup = document.getElementById('use-account-popup');
    for (i in users) {
        var item = document.createElement("menuitem");
        item.setAttribute('label', users[i].getAccountName());
        item.setAttribute('value', users[i].getUsername());
        var node = postAsPopup.appendChild(item);
        if (users[i].getUsername() == defaultUser) {
            document.getElementById('use-account-list').selectedItem = item;
        }
    }

    extendedModified();

    if (nsPreferences.getBoolPref(globals.deliciousService.useExperimentalKey, true) == true) {
      displayTags(globals.userManager.getDefaultUser().tags.getTagNamesSorted(), "tags-your", "You have no tags defined.", "tags-your-row", g_popularTags.concat(g_recommendedTags));
      if (globals.cookies.managementEnabled) {
        showLoadingStatus("tags-recommended");
        showLoadingStatus("tags-popular");
	    loadExperimentalFields(url, description);
      } else {
	    hideElement("tags-recommended-row");
	    hideElement("tags-popular-row");
	  }
	} else {
	    hideElement("tags-recommended-row");
	    hideElement("tags-your-row");
	    hideElement("tags-popular-row");
	}
}

function hideElement(id) {
    document.getElementById(id).hidden = true;
}

// Globals, needed to retain some info through the asynchronous loading process.
var g_url = '';
var g_username = '';
var g_password = '';
var g_description = '';
var g_recommendedTags = [];
var g_popularTags = [];
var gExperimentalLoaded = false;

// Globals from the caller. Needs to contain deliciousService and userManager objects.
var globals;

/*
 * Load del.icio.us' experimental post page, parse tags from it and populate
 * the dialog's "experimental" field with them.
 */
function loadExperimentalFields(url, description)
{
    if (gExperimentalLoaded == true) return;
    
   	g_username = globals.deliciousService.getUsername();
    g_password = globals.deliciousService.getPassword();
   	if (g_username.length == 0 || g_password.length == 0) {
   	    window.close();
   	    return;
   	}
    g_url = url;
    g_description = description;
    
	var queryURL = "http://del.icio.us/new/" + encodeURIComponent(g_username) +
		"?v=2&url=" + encodeURIComponent(g_url) +
		"&title=" + encodeURIComponent(g_description);
    globals.cookies.turnCookiesOn();
    asyncGetURL(queryURL, g_username, g_password, false, experimentalAsyncGetURLRequester);
}

//
// Callback for the asynchronous getting of user's tags.
//
function processTags(xml)
{
    if (xml == null) {
        clearLabelsFromElement("tags-your");
        var r = document.getElementById("tags-your");
        var label = document.createElement("label");
        label.setAttribute("value", "Failed getting your tags from del.icio.us. Please retry.");
        label.setAttribute("style", "font-style: italic;");
        r.appendChild(label);
        return;
    }
    var root = xml.documentElement;
    var tags = root.getElementsByTagName("tag");
    var tagsYour = new Array;
    for (var i = 0; i < tags.length; i++) {
        tagsYour.push(tags[i].getAttribute("tag"));
    }
    displayTags(tagsYour, "tags-your", "You have no tags defined.", "tags-your-row", g_popularTags.concat(g_recommendedTags));
}

var experimentalAsyncGetURLRequester = {
    // Process the asynchronous getting of the experimental fields info.
    asyncGetURLResponse: function(resp) {
        globals.cookies.restoreCookieState();
        if (resp != null) {
            gExperimentalLoaded = true;
            var tags = parseTagsFromResponse(resp, /var\s+tagRec\s*=\s*\[\s*(".*"|\s*,\s*)*\s*]/i);
            displayTags(tags, "tags-recommended", "No recommended tags found.", "tags-recommended-row", tags);
            g_recommendedTags = tags;
            tags = parseTagsFromResponse(resp, /var\s+tagPop\s*=\s*\[\s*(".*"|\s*,\s*)*\s*]/i);
            displayTags(tags, "tags-popular", "No popular tags found.", "tags-popular-row", tags);
            g_popularTags = tags;
        }
    },

    // Process error.
    asyncGetURLError: function(error) {
        globals.cookies.restoreCookieState();
        return;
    }
};

//
// Parse tags from the experimental response.
//
function parseTagsFromResponse(resp, regexp) {
    var tagmatch = resp.match(regexp);
    var tags = [];
    if (tagmatch && tagmatch[1] != null) {
        var tagarray = tagmatch[1].replace(/"/g, '')
        tags = tagarray.split(/\s*,\s*/);
    }
    return tags;
}

//
// Display tags or give a note if none are found. Emphasize given tags.
//
function displayTags(tags, elementId, noneFoundMessage, rowId, tagsToEmphasize) {
    writeTagsToDialogElement(tags, elementId);
    emphasizeTags(tagsToEmphasize);
    var r = document.getElementById(elementId);
    if (tags.length == 0) {
        var label = document.createElement("label");
        label.setAttribute("value", noneFoundMessage);
        label.setAttribute("style", "font-style: italic;");
        r.appendChild(label);
    } else {
        document.getElementById(rowId).setAttribute("flex", tags.length);
    }
}

//
// Indicate an error in loading the experimental fields.
//
function showExperimentalError(explanation, exception)
{
    var r = document.getElementById("tags-recommended");
    r.value = explanation + exception;
    r = document.getElementById("tags-popular");
    r.value = explanation + exception;
}    

//
// Write text into a tag field indicating that the field is being updated.
//
function showLoadingStatus(elementId)
{
    clearLabelsFromElement(elementId);
    var label = document.createElement("label");
    label.setAttribute("value", "Loading tags from del.icio.us...");
    label.setAttribute("style", "font-style: italic;");
    document.getElementById(elementId).appendChild(label);
}

//
// Clear any existing tags or text (labels) from a tag field.
//
function clearLabelsFromElement(elementId)
{
    var element = document.getElementById(elementId);
    var labels = element.getElementsByTagName('label');
    for (var i = labels.length - 1; i >= 0; --i) {
        element.removeChild(labels.item(i));
    }
}

/**
 * Store tags to a XULElement.
 */
function writeTagsToDialogElement(tags, elementId)
{
    clearLabelsFromElement(elementId);
    var element = document.getElementById(elementId);
    for (i in tags) {
        var label = document.createElement("label");
        label.setAttribute("value", tags[i]);
        label.setAttribute("onclick", "tagClicked(this);");
        element.appendChild(label);
    }
}

/**
 * Emphasize given tags in the user's tag list.
 */
function emphasizeTags(tags)
{
    var nodelist = document.getElementById('tags-your').childNodes;
    for (i in tags) {
        for (var j = nodelist.length - 1; j >= 0; j--) {
            if (nodelist.item(j).getAttribute('value') == tags[i]) {
                nodelist.item(j).setAttribute("style", "color: red;");
            }
        }
    }
}

/**
 * Switch clicked tags on or off.
 */
function tagClicked(tag)
{
    var tags = document.getElementById("tags").value.split(' ');
    var present = false;
    for (t in tags) {
        if (tags[t] == tag.value) {
            tags.splice(t,1);
            present = true;
        }
    }
    if (!present) {
        tags.push(tag.value);
    }
    document.getElementById("tags").value = tags.join(' ');
}

/**
 * Change the account to post to a different one.
 */
function changeAccount(event)
{
    clearLabelsFromElement("tags-your");
    globals.userManager.setDefaultUser(event.target.value);
//	loadExperimentalFields(document.getElementById("url").value, document.getElementById("description").value);
    if (nsPreferences.getBoolPref(globals.deliciousService.useExperimentalKey, true) == true) {
      displayTags(globals.userManager.getDefaultUser().tags.getTagNamesSorted(), "tags-your", "You have no tags defined.", "tags-your-row", g_popularTags.concat(g_recommendedTags));
    }
}

/**
 * Executed when user confirms the "post link" dialog box.
 */
function accept()
{
    var url = document.getElementById("url").value;
    var description = document.getElementById("description").value;
    var extended = document.getElementById("extended").value;
    var tags = document.getElementById("tags").value;
    var date = new Date();

    globals.deliciousService.postURL(url, description, extended, tags, date.toISO8601String(), acceptAsyncGetURLRequester);
    document.documentElement.getButton("accept").disabled = true;
    document.getElementById("status-row").hidden = false;
    document.getElementById("status-text").value = "Posting data."

    tags = tags.match(/\s*(\S.*\S)\s*/);
    if (tags != null) {
        var userTags = globals.userManager.getDefaultUser().tags;
        userTags.addTags(tags[1].split(/\s+/));
        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observerService.notifyObservers(null,"ext:delicious:tagxmlForDefaultUser", userTags.asXml(true));
    }

    return false;
}

/**
 * Executed when the user cancels the post dialog.
 */
function cancelled()
{
    globals.cookies.restoreCookieState();
    return true;
}

var acceptAsyncGetURLRequester = {
    // Executed when the URL has been posted.
    asyncGetURLResponse: function(resp)
    {
        if (resp != null) {
            document.getElementById("postsDialog").cancelDialog();
        } else {
            document.documentElement.getButton("accept").disabled = false;
            document.getElementById("status-text").value = "Posting failed. Click ok to retry or cancel to exit."
        }
    },

    // Process error.
    asyncGetURLError: function(error) {
        return;
    }
};

/**
 * Format a date as required to make a del.icio.us post (the &dt= parameter).
 * This uses ISO 8601 format: "CCYY-MM-DDThh:mm:ssZ"
 * See here for details: http://www.cl.cam.ac.uk/~mgk25/iso-time.html
 * Example: 1984-09-01T14:21:31Z
 */
Date.prototype.toISO8601String = function()
{
    var str;
    
    // date
    var year = this.getUTCFullYear().toString();
    var month = (this.getUTCMonth()+1).toString();
    month = month.length < 2 ? ("0" + month) :  month;
    
    var day = this.getUTCDate().toString();
    day = day.length < 2 ? ("0" + day) : day;
    
    var hours = this.getUTCHours().toString();
    hours = hours.length < 2 ? ("0" + hours) : hours;
    
    var minutes = this.getUTCMinutes().toString();
    minutes = minutes.length < 2 ? ("0" + minutes) : minutes;
    
    var seconds = this.getUTCSeconds().toString();
    seconds = seconds.length < 2 ? ("0" + seconds) : seconds;
    
    return year + "-" + month + "-" + day + "T" + hours + ":" + minutes + ":" + seconds + "Z";
}

/**
 * Parse an ISO 8601 formatted string into a Date object.
 */
Date.prototype.fromISO8601String = function(str)
{
    str = str.replace("Z","");
    var parts = str.split("T");
    var dateArr = parts[0].split("-");
    var timeArr = parts[1].split(":");
    
    this.setUTCFullYear(dateArr[0]);
    this.setUTCMonth(dateArr[1]-1);
    this.setUTCDate(dateArr[2]);
    this.setUTCHours(timeArr[0]);
    this.setUTCMinutes(timeArr[1]);
    this.setUTCSeconds(timeArr[2]);
}

/**
 * Update extended field's length info when the field is changed.
 */
function extendedModified()
{
    length = document.getElementById("extended").textLength;
    document.getElementById("extended-label").value = length + " / 255";
}
